module Main where

import Data.Set as Set
import System.IO (hSetBuffering,stdout, BufferMode(..),hPutStrLn,stderr)
import System.Exit(exitWith, ExitCode(..),exitSuccess)

----------------------------------------------------------------------

main = do
  hSetBuffering stdout LineBuffering
  simulation [s0]

----------------------------------------------------------------------

-- | Prints the state using the TXT format.
state2txt :: Int -> State -> String
state2txt number state
  | number > modelSize = ""
  | otherwise = (show (agents !! (number - 1))) ++ ": ("
  ++ (show (takeam number state)) ++ ","
  ++ (show (takepc number state)) ++ ","
  ++ (show (toList (takeci number state))) ++ ","
  ++ (takepvString number state) ++ ")  "
  ++ (state2txt (number + 1) state)

----------------------------------------------------------------------

-- | Prints a numbered list of states.
showNumberedStates :: Int -> [State] -> String
showNumberedStates _ [] = "-------\nChoose state:"
showNumberedStates lp (x:xs) = show lp ++ ". " ++ (state2txt 1 x)
  ++ "\n" ++ showNumberedStates (lp + 1) xs

----------------------------------------------------------------------

-- | Prints a numbered list of transitions.
showNumberedTrans :: Int -> [TTransition] -> String
showNumberedTrans _ [] = "-------\nThere is no enable transitions. Hit return to quit!"
showNumberedTrans lp [x] = show lp ++ ". " ++ show x ++ "\n-------\nChoose transition:"
showNumberedTrans lp (x:xs) = show lp ++ ". " ++ show x
  ++ "\n" ++ showNumberedTrans (lp + 1) xs

----------------------------------------------------------------------

-- | Simulate the given model interactively.
simulation :: [State] -> IO ()
simulation [] = putStrLn "--EOF--"
simulation states = do
  putStrLn $ ""
  putStrLn $ showNumberedStates 1 states
  chosenState <- getLine
  let cState = states!!((read chosenState :: Int) - 1)
  let transitions = enableTransitions cState
  putStrLn $ ""
  putStrLn $ showNumberedTrans 1 transitions
  chosenTransition <- getLine
  if chosenTransition == "q"
      then exitSuccess
      else simulation $ fire (transitions!!((read chosenTransition :: Int) - 1)) cState

----------------------------------------------------------------------
