-- Printing LTS to DOT --

----------------------------------------------------------------------

-- | Prints the LTS graph using the DOT format.
printDOT :: [Node] -> String
printDOT ltsGraph = "digraph G {\n\t rankdir=TB;\n"
  ++ insertEscapeChar (dumpDOTNodes ltsGraph) ++ dumpDOTTrans ltsGraph ++ "}\n"

----------------------------------------------------------------------

-- | Prints the LTS graph arcs using the DOT format.
dumpDOTTrans :: [Node] -> String
dumpDOTTrans [] = []
dumpDOTTrans (node:nodes) = dumpSingleTrans node ++ dumpDOTTrans nodes

----------------------------------------------------------------------

-- | Prints list of arcs for the given node (DOT format).
dumpSingleTrans :: Node -> String
dumpSingleTrans (number,_,arcs) = dumpTR (number, arcs)

----------------------------------------------------------------------

-- | Recursive function for printing a list of arcs for the given node.
dumpTR :: (Int,[(TTransition,Int)]) -> String
dumpTR (_,[]) = []
dumpTR (source,(transition,destination):arcs)
  = "\t" ++ (show source) ++ " -> " ++ (show destination)
  ++ " [label = \"" ++ (show transition) ++ "\"];\n"
  ++ dumpTR (source,arcs)

----------------------------------------------------------------------

-- | Inserts escape characters to the string representaion of graph nodes (DOT format).
insertEscapeChar :: String -> String
insertEscapeChar (x:xs)
  | isPrefix "label = \"(" (x:xs) = "label = \"(" ++ (insertEscapeChar (Prelude.drop 10 (x:xs)))
  | isPrefix ")\" shape=box" (x:xs) = ")\" shape=box" ++ (insertEscapeChar (Prelude.drop 12 (x:xs)))
  | x =='\"' = "\\\"" ++ (insertEscapeChar xs)
  | otherwise = x : (insertEscapeChar xs)
insertEscapeChar [] = []

----------------------------------------------------------------------

-- | Prints the LTS graph nodes using the DOT format.
dumpDOTNodes :: [Node] -> String
dumpDOTNodes [] = []
dumpDOTNodes ((number,state,trans):nodes)
  = "\t" ++ (show number)
  ++ " [label = \"" ++ "(" ++ (show number) ++ ")" ++ (dumpDOTState 1 state)
  ++ "\" shape=box];\n" ++ dumpDOTNodes nodes

----------------------------------------------------------------------

-- | Prints the state using the DOT format.
dumpDOTState :: Int -> State -> String
dumpDOTState number state
  | number > modelSize = ""
  | otherwise = "\\n" ++ (show (agents !! (number - 1))) ++ ": ("
  ++ (show (takeam number state)) ++ ","
  ++ (show (takepc number state)) ++ ","
  ++ (show (toList (takeci number state))) ++ ","
  ++ (takepvString number state) ++ ")"
  ++ (dumpDOTState (number + 1) state)

----------------------------------------------------------------------
