-- Printing time-LTS (alpha^1) to DOT --

----------------------------------------------------------------------

-- | Prints the LTS graph using the DOT format.
printDOT :: [Node] -> String
printDOT ltsGraph = "digraph G {\n\t rankdir=TB;\n"
  ++ insertEscapeChar (dumpDOTNodes ltsGraph) ++ dumpDOTTrans ltsGraph ++ "}\n"

----------------------------------------------------------------------

-- | Inserts escape characters to the string representaion of graph nodes (DOT format).
insertEscapeChar :: String -> String
insertEscapeChar (x:xs)
  | isPrefix "label = \"(" (x:xs) = "label = \"(" ++ (insertEscapeChar (Prelude.drop 10 (x:xs)))
  | isPrefix ")\" shape=box" (x:xs) = ")\" shape=box" ++ (insertEscapeChar (Prelude.drop 12 (x:xs)))
  | x =='\"' = "\\\"" ++ (insertEscapeChar xs)
  | otherwise = x : (insertEscapeChar xs)
insertEscapeChar [] = []

----------------------------------------------------------------------

-- | Prints the LTS graph nodes using the DOT format.
dumpDOTNodes :: [Node] -> String
dumpDOTNodes [] = []
dumpDOTNodes ((number,state,queue,time,arcs):nodes)
  = "\t" ++ (show number)
  ++ " [label = \"" ++ "(" ++ (show number) ++ ")" ++ (dumpDOTState 1 state queue)
  ++ "\\n" ++ (show (queue2print)) ++ ", (" ++ (show time)       -- We drop the first element because this is the active agent.
  ++ ")\" shape=box];\n" ++ dumpDOTNodes nodes
  where queue2print = if queue /= [] then tail queue else []

----------------------------------------------------------------------

-- | Prints the state using the DOT format. Maps X to R for non-running agents.
dumpDOTState :: Int -> State -> [(Int,Int)] -> String
dumpDOTState number state queue
  | number > modelSize = ""
  | number == activeNumber = "\\n" ++ (show (agents !! (number - 1))) ++ ": ("
    ++ (show (takeam number state)) ++ ","
    ++ (show (takepc number state)) ++ ","
    ++ (show (toList (takeci number state))) ++ ","
    ++ (takepvString number state) ++ ")"
    ++ (dumpDOTState (number + 1) state queue)  
  | number /= activeNumber && (takeam number state) == X
    = "\\n" ++ (show (agents !! (number - 1))) ++ ": (R,"
    ++ (show (takepc number state)) ++ ","
    ++ (show (toList (takeci number state))) ++ ","
    ++ (takepvString number state) ++ ")"
    ++ (dumpDOTState (number + 1) state queue)  
  | otherwise = "\\n" ++ (show (agents !! (number - 1))) ++ ": ("
    ++ (show (takeam number state)) ++ ","
    ++ (show (takepc number state)) ++ ","
    ++ (show (toList (takeci number state))) ++ ","
    ++ (takepvString number state) ++ ")"
    ++ (dumpDOTState (number + 1) state queue)
  where activeNumber = if queue /= [] then (fst (head queue)) else -1
        
----------------------------------------------------------------------

-- | Prints the LTS graph arcs using the DOT format.
dumpDOTTrans :: [Node] -> String
dumpDOTTrans [] = []
dumpDOTTrans (node:nodes) = dumpSingleTrans node ++ dumpDOTTrans nodes

----------------------------------------------------------------------

-- | Prints list of arcs for the given node (DOT format).
dumpSingleTrans :: Node -> String
dumpSingleTrans (number,_,_,_,arcs) = dumpTR (number,arcs)

----------------------------------------------------------------------

-- | Recursive function for printing a list of arcs for the given node.
dumpTR :: (Int,[(TTransition,Int,Int)]) -> String
dumpTR (_,[]) = []
dumpTR (source,(transition,time,destination):arcs)
  = "\t" ++ (show source) ++ " -> " ++ (show destination)
  ++ " [label = \"" ++ (show transition) ++ "/" ++ (show time) ++ "\"];\n"
  ++ dumpTR (source,arcs)

----------------------------------------------------------------------

